@@ -72,7 +72,9 @@
 /*****************************************************************************
  *  This is a rewrite/clone of the arpa/tftp.h file for systems without it.  *
  *****************************************************************************/
+
 #define SEGSIZE 512 /* data segment size */
+#define PKTSIZE (SEGSIZE + 4)  /* SEGSIZE defined in arpa/tftp.h */
 
 #if defined(__GNUC__) && ((__GNUC__ >= 3) || \
   ((__GNUC__ == 2) && defined(__GNUC_MINOR__) && (__GNUC_MINOR__ >= 7)))
@@ -108,8 +110,6 @@ struct tftphdr {
  *                      STRUCT DECLARATIONS AND DEFINES                      *
  *****************************************************************************/
 
-#define PKTSIZE (SEGSIZE + 4)  /* SEGSIZE defined in arpa/tftp.h */
-
 struct testcase {
   char *buffer;   /* holds the file data to send to the client */
   size_t bufsize; /* size of the data in buffer */
@@ -454,7 +454,7 @@ static ssize_t write_behind(struct testcase *test, int convert)
     c = *p++;                     /* pick up a character */
     if(prevchar == '\r') {        /* if prev char was cr */
       if(c == '\n')               /* if have cr,lf then just */
-        lseek(test->ofile, -1, SEEK_CUR); /* smash lf on top of the cr */
+        curl_lseek(test->ofile, -1, SEEK_CUR); /* smash lf on top of the cr */
       else if(c == '\0')          /* if have cr,nul then */
         goto skipit;              /* just skip over the putc */
       /* else just fall through and allow it */
@@ -548,7 +548,7 @@ static int tftpd_parse_servercmd(struct testcase *req)
   }
   else {
     char *orgcmd = NULL;
-    char *cmd = NULL;
+    const char *cmd = NULL;
     size_t cmdsize = 0;
     int num = 0;
 
@@ -562,7 +562,7 @@ static int tftpd_parse_servercmd(struct testcase *req)
 
     cmd = orgcmd;
     while(cmd && cmdsize) {
-      char *check;
+      const char *check;
       if(sscanf(cmd, "writedelay: %d", &num) == 1) {
         logmsg("instructed to delay %d secs between packets", num);
         req->writedelay = num;
@@ -600,7 +600,7 @@ static int tftpd_parse_servercmd(struct testcase *req)
 static int validate_access(struct testcase *test,
                            const char *filename, unsigned short mode)
 {
-  char *ptr;
+  const char *ptr;
 
   logmsg("trying to get file: %s mode %x", filename, mode);
 
@@ -849,7 +849,7 @@ send_ack:
   write_behind(test, pf->f_convert);
   /* close the output file as early as possible after upload completion */
   if(test->ofile > 0) {
-    close(test->ofile);
+    curlx_close(test->ofile);
     test->ofile = 0;
   }
 
@@ -875,7 +875,7 @@ send_ack:
 abort:
   /* make sure the output file is closed in case of abort */
   if(test->ofile > 0) {
-    close(test->ofile);
+    curlx_close(test->ofile);
     test->ofile = 0;
   }
   return;
@@ -1305,7 +1305,7 @@ static int test_tftpd(int argc, char **argv)
 tftpd_cleanup:
 
   if(test.ofile > 0)
-    close(test.ofile);
+    curlx_close(test.ofile);
 
   if((peer != sock) && (peer != CURL_SOCKET_BAD))
     sclose(peer);
